<?php
/**
 * Cache.inc.php
 * code by strangecode :: www.strangecode.com :: this document contains copyrighted information
 *
 * Provides an API for storing a limited amount of data
 * intended to have a short lifetime in a user's session.
 *
 * @author  Quinn Comendant <quinn@strangecode.com>
 * @version 2.1
 * @since   2001
 */
 
// Flags.
define('CACHE_ALLOW_OVERSIZED', 1);

class Cache {

    // Namespace of this instance of Prefs.
    var $_ns;

    // Configuration parameters for this object.
    var $_params = array(
        
        // If false nothing will be cached or retreived. Useful for testing realtime data requests.
        'enabled' => true,

        // The maximum size in bytes of any one variable.
        'item_size_limit' => 4194304, // 4 MB
        
        // The maximum size in bytes before the cache will begin flushing out old items.
        'stack_size_limit' => 4194304, // 4 MB
        
        // The minimum items to keep in the cache regardless of item or cache size.
        'min_items' => 5,
    );
    
    /*
    * Constructor
    *
    * @access   public
    * @param    string  $namespace  This object will store data under this realm.
    * @author   Quinn Comendant <quinn@strangecode.com>
    * @version  1.0
    * @since    05 Jun 2006 23:14:21
    */
    function Cache($namespace='')
    {
        $this->_ns = '_cache' . $namespace;
        
        if (!isset($_SESSION[$this->_ns])) {
            $this->clear();
        }
    }

    /**
     * This method enforces the singleton pattern for this class.
     *
     * @return  object  Reference to the global Cache object.
     * @access  public
     * @static
     */
    function &getInstance($namespace='')
    {
        static $instances = array();

        if (!array_key_exists($namespace, $instances)) {
            $instances[$namespace] = new Cache($namespace);
        }

        return $instances[$namespace];
    }

    /**
     * Set (or overwrite existing) parameters by passing an array of new parameters.
     *
     * @access public
     * @param  array    $params     Array of parameters (key => val pairs).
     */
    function setParam($params)
    {
		$app =& App::getInstance();

        if (isset($params) && is_array($params)) {
            // Merge new parameters with old overriding only those passed.
            $this->_params = array_merge($this->_params, $params);
        } else {
            $app->logMsg(sprintf('Parameters are not an array: %s', $params), LOG_ERR, __FILE__, __LINE__);
        }
    }

    /**
     * Return the value of a parameter, if it exists.
     *
     * @access public
     * @param string $param        Which parameter to return.
     * @return mixed               Configured parameter value.
     */
    function getParam($param)
    {
		$app =& App::getInstance();
    
        if (isset($this->_params[$param])) {
            return $this->_params[$param];
        } else {
            $app->logMsg(sprintf('Parameter is not set: %s', $param), LOG_DEBUG, __FILE__, __LINE__);
            return null;
        }
    }

    /**
     * Stores a new variable in the session cache. The $key is is md5'ed
     * because if a key is numeric, the array_shift function
     * will reset the key to the next largest int key. Weird behavior I can't
     * understand. For example $cache["123"] will become $cache[0]
     *
     * @param str   $key        An identifier for the cached object.
     * @param mixed $var        The var to store in the session cache.
     * @param bool  $flags      If we have something really big that we
     *                          still want to cache, setting this to
     *                          CACHE_ALLOW_OVERSIZED allows this.
     * @return bool             True on success, false otherwise.
     */
    function set($key, $var, $flags=0)
    {
		$app =& App::getInstance();

        if (true !== $this->getParam('enabled')) {
            $app->logMsg(sprintf('Cache not enabled, not saving data.', null), LOG_DEBUG, __FILE__, __LINE__);
            return false;
        }

        $keyhash = md5($key);
        $var = serialize($var);
        $var_len = strlen($var);

        if ($var_len >= $this->getParam('item_size_limit')) {
            $app->logMsg(sprintf('Serialized variable (%s bytes) more than item_size_limit (%s bytes).', $var_len, $this->getParam('item_size_limit')), LOG_NOTICE, __FILE__, __LINE__);
            return false;
        }

        if ($flags & CACHE_ALLOW_OVERSIZED == 0 && $var_len >= $this->getParam('stack_size_limit')) {
            $app->logMsg(sprintf('Serialized variable (%s bytes) more than stack_size_limit (%s bytes).', $var_len, $this->getParam('stack_size_limit')), LOG_NOTICE, __FILE__, __LINE__);
            return false;
        }        

        // Remove any value already stored under this key.
        unset($_SESSION[$this->_ns][$keyhash]);

        // Continue to prune the cache if its size is greater than stack_size_limit, but keep at least min_items.
        while (strlen(serialize($_SESSION[$this->_ns])) + $var_len >= $this->getParam('stack_size_limit') && sizeof($_SESSION[$this->_ns]) >= $this->getParam('min_items')) {
            array_shift($_SESSION[$this->_ns]);
        }

        // Save this value under the specified key.
        $_SESSION[$this->_ns][$keyhash] =& $var;

        if ($var_len >= 1024000) {
            $app->logMsg(sprintf('Successfully cached oversized variable (%s bytes).', $var_len), LOG_DEBUG, __FILE__, __LINE__);
        }

        return true;
    }

    /**
     * Retrives an object from the session cache and returns it unserialized.
     * It also moves it to the top of the stack, which makes it such that the
     * cache flushing mechanism of putCache deletes the oldest referenced items
     * first.
     *
     * @param string $key  The key for the datum to retrieve.
     * @return mixed          The requested datum, or false on failure.
     */
    function get($key)
    {
        $app =& App::getInstance();
        
        if (true !== $this->getParam('enabled')) {
            $app->logMsg(sprintf('Cache not enabled, not getting data.', null), LOG_DEBUG, __FILE__, __LINE__);
            return false;
        }

        $keyhash = md5($key);
        if (isset($_SESSION[$this->_ns][$keyhash])) {
            $app->logMsg(sprintf('Retreiving %s from cache.', $key), LOG_DEBUG, __FILE__, __LINE__);
            // Move the accessed cached datum to the top of the stack. Maybe somebody knows a better way to do this?
            $tmp =& $_SESSION[$this->_ns][$keyhash];
            unset($_SESSION[$this->_ns][$keyhash]);
            $_SESSION[$this->_ns][$keyhash] =& $tmp;
            // Return the unserialized datum.
            return unserialize($_SESSION[$this->_ns][$keyhash]);
        } else {
            return false;
        }
    }

    /**
     * Tells you if the object is cached.
     *
     * @param string $key  The key of the object to check.
     * @return bool           The return from isset().
     */
    function exists($key)
    {
        if (true !== $this->getParam('enabled')) {
            return false;
        }

        $keyhash = md5($key);
        return array_key_exists($keyhash, $_SESSION[$this->_ns]);
    }

    /**
     * Tells you if the object is cached.
     *
     * @param string $key  The key of the object to check.
     * @return bool           The return from isset().
     */
    function delete($key)
    {
        $keyhash = md5($key);
        unset($_SESSION[$this->_ns][$keyhash]);
    }
    
    /*
    * Delete all existing items from the cache.
    *
    * @access   public
    * @author   Quinn Comendant <quinn@strangecode.com>
    * @version  1.0
    * @since    05 Jun 2006 23:51:34
    */
    function clear()
    {
        $_SESSION[$this->_ns] = array();
    }

// END Cache
}

?>